<?php
namespace Classes\Attendance;

use Classes\Attendance\Manager as AManager;
use Classes\Login\Manager as LManager;
use Classes\Registration\Manager as RManager;
use Classes\DateTimeUtil;
use Classes\Paginator;
use Classes\StaticFunctions;
use Classes\StaticFunctions as Statics;

class Facade
{
    private Paginator $paginator;
    private AManager $attendanceManager;

    public function __construct(AManager $attendanceManager, Paginator $paginator)
    {
        $this->attendanceManager = $attendanceManager;
        $this->paginator = $paginator;
    }

    public function getAttendanceForUser(int $page, int $isAdminPage, bool $generate = false)
    {
        $userAttendanceQuery = $this->attendanceManager->getAllForUser(Statics::getUserID());

        $limit = $this->paginator->getLimit();
        $count = $userAttendanceQuery->count();
        $this->paginator->setQueryLimit($userAttendanceQuery, $limit, $this->paginator->getOffset($limit, $page));

        $userAttendanceData = $userAttendanceQuery->fetchAll();

        foreach ($userAttendanceData as $key => $attendance)
        {
            $dateTimeUtil = new DateTimeUtil(new \DateTime($attendance[AManager::CREATE_DT]));
            $userAttendanceData[$key][AManager::CREATE_DT] = $dateTimeUtil->getDateTimeFormat();
            $userAttendanceData[$key][AManager::WORK_FROM] = DateTimeUtil::getTimeFormat($attendance[AManager::WORK_FROM]);
            $userAttendanceData[$key][AManager::WORK_TO] = DateTimeUtil::getTimeFormat($attendance[AManager::WORK_TO]);
        }

        if ($generate)
        {
            return Statics::renderLatte('www/user/tables/attendance.latte', [
                    'pages' =>  $this->paginator->getNumberOfPages($count, $limit),
                    'attendances' => $userAttendanceData,
                    'page' => $page,
                    'isAdminPage' => $isAdminPage,
                    'programs' => AManager::PROGRAM_DICTIONARY
                ]
            );
        } else {
            return [
                'pages' =>  $this->paginator->getNumberOfPages($count, $limit),
                'data' => $userAttendanceData
            ];
        }
    }

    public function getAttendanceForAdmin(int $page, int $isAdminPage, bool $generate = false)
    {
        $attendanceQuery = $this->attendanceManager->getAllForCoordinator(Statics::getOrganization());

        $limit = $this->paginator->getLimit();
        $count = $attendanceQuery->count();
        $this->paginator->setQueryLimit($attendanceQuery, $limit, $this->paginator->getOffset($limit, $page));

        $attendanceData = $attendanceQuery->fetchAll();

        if ($generate)
        {
            return Statics::renderLatte('www/admin/tables/attendanceMain.latte', [
                    'pages' =>  $this->paginator->getNumberOfPages($count, $limit),
                    'attendances' => $attendanceData,
                    'page' => $page,
                    'programs' => AManager::PROGRAM_DICTIONARY,
                    'isAdminPage' => $isAdminPage
                ]
            );
        } else {
            return [
                'pages' =>  $this->paginator->getNumberOfPages($count, $limit),
                'data' => $attendanceData
            ];
        }
    }

    public function getAttendanceForReport(int $page, array $filters = [], int $working = 0, bool $generate = false)
    {
        $attendanceQuery = $this->attendanceManager->getAllForCoordinator(Statics::getOrganization(), $filters);

        $this->paginator->setLimit(50);
        $limit = $this->paginator->getLimit();
        $count = $attendanceQuery->count();
        $this->paginator->setQueryLimit($attendanceQuery, $limit, $this->paginator->getOffset($limit, $page));

        $attendanceData = $attendanceQuery->fetchAll();

        foreach ($attendanceData as $attendanceKey => $attendance)
        {
            $attendanceData[$attendanceKey][AManager::WORK_FROM] = DateTimeUtil::getTimeFormat($attendance[AManager::WORK_FROM]);
            $attendanceData[$attendanceKey][AManager::WORK_TO] = DateTimeUtil::getTimeFormat($attendance[AManager::WORK_TO]);
        }

        if ($generate)
        {
            return Statics::renderLatte('www/admin/tables/attendanceReport.latte', [
                    'pages' =>  $this->paginator->getNumberOfPages($count, $limit),
                    'attendances' => $attendanceData,
                    'page' => $page,
                    'programs' => AManager::PROGRAM_DICTIONARY,
                    'isWorkingHour' => $working,
                    'months' => \Classes\DateTimeUtil::getAllMonths()
                ]
            );
        } else {
            return [
                'pages' =>  $this->paginator->getNumberOfPages($count, $limit),
                'data' => $attendanceData
            ];
        }
    }

    public function getWorkingHourData(int $page, array $filters = [], bool $generate = false)
    {
        $sumFilter = Statics::getWorkingHourReportType();
        $count = $limit = 1;
        if ($sumFilter)
        {
            $data = $this->attendanceManager->getAllForCoordinator(StaticFunctions::getOrganization(), $filters, true);
        } else {
            $dataQuery = $this->attendanceManager->getAllForCoordinator(StaticFunctions::getOrganization(), $filters);

            $this->paginator->setLimit(25);
            $limit = $this->paginator->getLimit();
            $count = $dataQuery->count();
            $this->paginator->setQueryLimit($dataQuery, $limit, $this->paginator->getOffset($limit, $page));

            $data = $dataQuery->fetchAll();
        }

        $userNameGenerated =  sprintf('%s %s (%s)',
            $data[0][RManager::FIRST_NAME], $data[0][RManager::SUR_NAME], $data[0][LManager::USER_NAME]);
        $sum = 0;
        foreach ($data as $key => $attendance)
        {
            $data[$key][AManager::WORK_FROM] = DateTimeUtil::getTimeFormat($attendance[AManager::WORK_FROM]);
            $data[$key][AManager::WORK_TO] = DateTimeUtil::getTimeFormat($attendance[AManager::WORK_TO]);
            $data[$key][LManager::USER_NAME] = $userNameGenerated;
            $sum = $sum + $attendance[AManager::WORK_HOURS];
        }

        $sumData = [
            'sum' => $sum,
            AManager::PROGRAM => AManager::PROGRAM_DICTIONARY[$data[0][AManager::PROGRAM]],
            AManager::YEAR => $data[0][AManager::YEAR],
            AManager::MONTH => DateTimeUtil::getMonth($data[0][AManager::MONTH]),
            LManager::USER_NAME =>$userNameGenerated
        ];

        if ($generate)
        {
            return Statics::renderLatte('www/admin/tables/hoursReport.latte', [
                    'pages' =>  $this->paginator->getNumberOfPages($count, $limit),
                    'data' => $data,
                    'sumData' => $sumData,
                    'page' => $page,
                    'programs' => AManager::PROGRAM_DICTIONARY,
                    'months' => \Classes\DateTimeUtil::getAllMonths(),
                    'years' => $this->attendanceManager->getAllYears(true),
                    'isWorkingHour' => 1,
                    'sum' => $sumFilter,
                    'filter' => $this->getActiveFilter($filters)
                ]
            );
        } else {
            if ($sumFilter)
            {
                return $sumData;
            } else {
                return $data;
            }
        }
    }

    private function getActiveFilter(array $filters) : ?int
    {
        if ($filters['month'] != "")
        {
            return 1;
        }
        if ($filters['user'] != "")
        {
            return 2;
        }
        if ($filters['year'] != "")
        {
            return 3;
        }

        return null;
    }

    public function getApiAttendance(int $userID)
    {
        $userAttendanceData = $this->attendanceManager->getAllForUser($userID, false);

        foreach ($userAttendanceData as $key => $attendance)
        {
            $dateTimeUtil = new DateTimeUtil(new \DateTime($attendance[AManager::DATE]));
            $userAttendanceData[$key][AManager::DATE] = $dateTimeUtil->getDateFormat();
            $userAttendanceData[$key][AManager::WORK_FROM] = DateTimeUtil::getTimeFormat($attendance[AManager::WORK_FROM]);
            $userAttendanceData[$key][AManager::WORK_TO] = DateTimeUtil::getTimeFormat($attendance[AManager::WORK_TO]);
            $userAttendanceData[$key][AManager::PROGRAM] = AManager::PROGRAM_DICTIONARY[$attendance[AManager::PROGRAM]];
        }

        return $userAttendanceData;
    }
}