<?php
namespace Classes\Attendance;

use \Classes\Login\Manager as LManager;
use \Classes\Registration\Manager as RManager;
use Classes\StaticFunctions;
use Dibi\Fluent;

class Manager extends \Classes\Dibi
{
    const TABLE = 'tAttendance';
    const PRIMARY = 'id';

    const FK_USER_ID = 'FK_userID';
    const PROGRAM = 'program';
    const MONTH = 'month';
    const WORK_HOURS = 'workHours';
    const WORK_FROM = 'workFrom';
    const WORK_TO = 'workTo';
    const DESCRIPTION = 'description';
    const DATE = 'date';
    const YEAR = 'year';
    const CREATE_DT = 'createDT';
    const STATE = 'state';

    const PROGRAM_DICTIONARY = [
        0 => 'Velký kamarád',
        1 => 'Babička - Dědeček',
        2 => 'Kamarád pro skupinu',
        3 => 'Dobrovolník v sociálních službách',
        4 => 'Dobrovolník v nemocnici'
    ];

    const PROGRAM_DICTIONARY_REVERSE = [
        'Velký kamarád' => 0,
        'Babička - Dědeček'=> 1,
        'Kamarád pro skupinu'=> 2,
        'Dobrovolník v sociálních službách'=> 3,
        'Dobrovolník v nemocnici'=> 4
    ];

    const PROGRAM_DICTIONARY_REVERSE_OLD = [
        'PětP' => 0,
        '3G' => 1,
        'B612' => 2,
        'K2' => 3,
        'N80' => 4,
    ];

    public static function getProgramName(int $program) : string
    {
        return self::PROGRAM_DICTIONARY[$program];
    }

    public function getAllForUser(int $userID, bool $onlyQuery = true)
    {
        $query = $this->db->select(self::ALL);
        $query->from(self::TABLE);
        $query->where('%n = %i', self::FK_USER_ID, $userID);
        $query->orderBy(self::CREATE_DT)->desc();

        if ($onlyQuery)
        {
            return $query;
        } else {
            return $this->fetchAll($query);
        }
    }

    public function getAllForCoordinator(int $organizationID, array $filter = [], bool $data = false)
    {
        $query = $this->db->select('a.*');
        $query->select('l.%n', LManager::USER_NAME);
        $query->select('r.%n', RManager::FIRST_NAME);
        $query->select('r.%n', RManager::SUR_NAME);
        $query->from(self::TABLE)->as('a');
        $query->leftJoin(LManager::TABLE)->as('l')->on('a.%n = l.%n',
            self::FK_USER_ID, LManager::PRIMARY);
        $query->leftJoin(RManager::TABLE)->as('r')->on('l.%n = r.%n',
            LManager::PRIMARY, RManager::FK_LOGIN_ID);

        if (array_key_exists('program', $filter) && !is_null($filter['program']))
        {
            $query->where('a.%n = %i', self::PROGRAM, $filter['program']);
        }

        if (array_key_exists('month', $filter) && !is_null($filter['month']))
        {
            $query->where('a.%n = %i', self::MONTH, $filter['month']);
        }

         if (array_key_exists('user', $filter) && !is_null($filter['user']))
        {
            $query->where('a.%n = %i', self::FK_USER_ID, $filter['user']);
        }

        if (array_key_exists('year', $filter) && !is_null($filter['year']))
        {
            $query->where('a.%n = %i', self::YEAR, $filter['year']);
        }

        $query->where('l.%n = %i', LManager::FK_ORGANIZATION_ID, $organizationID);
        $query->orderBy(self::STATE);
        $query->orderBy(self::DATE)->desc();

        if ($data)
        {
            return $this->fetchAll($query);
        } else {
            return $query;
        }
    }

    public function getAllForReport(int $organizationID, array $filter, bool $data = false)
    {
        $query = $this->db->select('a.*');
        $query->select('r.%n', RManager::FIRST_NAME);
        $query->select('r.%n', RManager::SUR_NAME);
        $query->select('l.%n', LManager::USER_NAME);
        $query->from(self::TABLE)->as('a');
        $query->leftJoin(LManager::TABLE)->as('l')->on('a.%n = l.%n',
            self::FK_USER_ID, LManager::PRIMARY);
        $query->leftJoin(RManager::TABLE)->as('r')->on('l.%n = r.%n',
            LManager::PRIMARY, RManager::FK_LOGIN_ID);
        $query->where('l.%n = %i', LManager::FK_ORGANIZATION_ID, $organizationID);
        $query->orderBy(self::STATE);
        $query->orderBy(self::DATE)->desc();


        if ($data)
        {
            return $this->fetchAll($query);
        } else {
            return $query;
        }
    }

    public function getAllUsers() : array
    {
        $query = $this->db->select(LManager::USER_NAME);
        $query->select(RManager::FIRST_NAME);
        $query->select(RManager::SUR_NAME);
        $query->select('a.%n', self::FK_USER_ID)->as('id');
        $query->from(self::TABLE)->as('a');
        $query->leftJoin(LManager::TABLE)->as('l')->on('a.%n = l.%n',
            self::FK_USER_ID, LManager::PRIMARY);
        $query->leftJoin(RManager::TABLE)->as('r')->on('l.%n = r.%n',
            LManager::PRIMARY, RManager::FK_LOGIN_ID);
        $query->where('l.%n = %i', LManager::FK_ORGANIZATION_ID, StaticFunctions::getOrganization());
        $query->where('l.%n != %i', LManager::PERMISSION, LManager::PERMISSION_GOD);
        $query->where('l.%n != %i', LManager::PRIMARY, StaticFunctions::getUserID());
        $query->groupBy(LManager::USER_NAME);
        $query->groupBy(RManager::FIRST_NAME);
        $query->groupBy(RManager::SUR_NAME);
        $query->groupBy('a.%n', self::FK_USER_ID);

        return $this->fetchAll($query);
    }

    public function getAllYears(bool $assoc = false) : array
    {
        $query = $this->db->select(self::YEAR);
        $query->from(self::TABLE)->as('a');
        $query->leftJoin(LManager::TABLE)->as('l')->on('a.%n = l.%n',
            self::FK_USER_ID, LManager::PRIMARY);
        $query->where('l.%n = %i', LManager::FK_ORGANIZATION_ID, StaticFunctions::getOrganization());
        $query->where('l.%n != %i', LManager::PERMISSION, LManager::PERMISSION_GOD);
        $query->where('l.%n != %i', LManager::PRIMARY, StaticFunctions::getUserID());
        $query->groupBy(self::YEAR);

        if ($assoc)
        {
            $return = [];
            foreach ($query->fetchAll() as $years)
            {
                $return[$years['year']] = $years['year'];
            }
            return $return;
        } else {
            return $this->fetchAll($query);
        }

    }

    public function deleteUserData(int $id) : void
    {
        $this->db->delete(self::TABLE)
        ->where('%n = %i', self::FK_USER_ID, $id)
        ->execute();
    }

    public function try($id, $isDesc, $needAddress)
    {
        $query = $this->db->select('*');
        $query->from('userTable');
        $query->select('userID')
            ->select(self::PRIMARY);

        if (!is_null($id))
        {
            $query->where('userID = %i', $id);
        }

        $query->where("userName LIKE '%user%'");
        $query->orderBy('userID');
        if ($isDesc)
        {
            $query->desc();
        }

        if($needAddress)
        {
            $query->leftJoin('adressTable')->as('at')->on('at.%n = %sql', 'addressID',
                $this->db->select('pt.addressID')
                ->from('personTable')->as('pt')
                ->leftJoin('userTable')->as('ut')->on('pt.FK_userID = ut.userID')
                ->where('ut.userID = %n', 'userID'));
        }
    }
}